//#define STARACCEPTANCE

#include <iostream>
#include <cstdlib>
#include <iomanip>
#include <fstream>
#include <cmath>
#include <ctime>
#include <iostream>
#include <string>

using namespace std;

#include "../../random/random.h"
CRandom *ranptr;
#include "parameter.cc"
#include "../../canonical/src/ressubs.h"
#include "balance.h"
double triangle(double m0,double m1,double m2);
void resonancedecay(Tpartinfo *&mother,int *nbodies, Tpartinfo **&daughter);
void GetParts(int &nparts,Tpartinfo **&newparts,Treslist &reslist,
	      CParameter &parm);
void Copypartinfo(Tpartinfo *targetpart,Tpartinfo *sourcepart);
void BjorkenBoost(int nparts,Tpartinfo **&newparts,CParameter &parm);
void CheckTotalCharge(int nparts,Tpartinfo **&newparts);
ifstream inputfile;

int main(int argc,char *argv[]){
  stringstream sst;
  string parfile;
  sst << argv[1];
  sst >> parfile;
  cout << "parfile=" << parfile << endl;
  
  CParameter parm(parfile);
  int nevents,ievent,nparts,ipart,nprint;
  Tbalance *balance;
  Tpartinfo **parts;
  Tresinfo *resinfoptr;
  Treslist reslist;
  ofstream outfile;
  balance=new Tbalance(&parm);
  ranptr=new CRandom(-1234);

  reslist.ReadResInfo();
  cout << "Will read phase space pt.s from " << parm.infilename << endl;
  inputfile.open(parm.infilename.c_str());
  parts=new Tpartinfo *[parm.AMAX];
  for(ipart=0;ipart<parm.AMAX;ipart++){
    parts[ipart]=new Tpartinfo();
  }

  inputfile >> nevents;
  cout << "Will process " << nevents << " ensembles" << endl;
  nprint=0;
  for(ievent=0;ievent<nevents;ievent++){
    GetParts(nparts,parts,reslist,parm);
    CheckTotalCharge(nparts,parts);
    BjorkenBoost(nparts,parts,parm);
    balance->AddParts(nparts,parts); 
    nprint=nprint+1;
    if(nprint==int(nevents/10)){
      cout << "Finished " << double(ievent+1)*100/double(nevents) 
	   << "%, Nplus=" << balance->Np << ", Nminus=" << balance->Nm << endl;
      nprint=0;
    }
  }
  balance->Calc();
  return 0;
}

void GetParts(int &nparts,Tpartinfo **&parts,Treslist &reslist,
	      CParameter &parm){
  Tpartinfo *mother;
  int nread,iread;
  int mothercode,ipart,alpha;
  
  mother=new Tpartinfo;

  inputfile >> nread;
  nparts=0;
  for(iread=0;iread<nread;iread++){
    inputfile >> mothercode >> mother->p[1] >> mother->p[2] 
	      >> mother->p[3];
    reslist.Getresinfoptr(mothercode,mother->resinfoptr);
    mother->p[0]=pow(mother->resinfoptr->mass,2);
    for(alpha=1;alpha<4;alpha++) mother->p[0]+=pow(mother->p[alpha],2);
    mother->p[0]=sqrt(mother->p[0]);
    Copypartinfo(parts[nparts],mother);
    nparts+=1;
    if(nparts>=parm.AMAX){
      cout << "parm.AMAX too small!!!\n";
      exit(1);
    }
  }

  delete mother;

}

void CheckTotalCharge(int nparts,Tpartinfo **&parts){
  int ipart,netq,netb,nets;
  netq=netb=nets=0;
  for(ipart=0;ipart<nparts;ipart++){
    parts[ipart]->resinfoptr->count+=1;
    netq+=parts[ipart]->resinfoptr->charge;
    netb+=parts[ipart]->resinfoptr->baryon;
    nets+=parts[ipart]->resinfoptr->strange;
  }
  if(netq!=0 || netb!=0 || nets !=0){
    cout << "netq =" << netq << ", netb=" << netb 
	 <<  "nets=" << nets << endl;
    for(ipart=0;ipart<nparts;ipart++){
      cout <<  parts[ipart]->resinfoptr->name << ","
	   <<	 parts[ipart]->resinfoptr->code << ","
	   << parts[ipart]->resinfoptr->charge << ","
	   << parts[ipart]->resinfoptr->baryon << ","
	   << parts[ipart]->resinfoptr->strange << endl;
    }
    exit(1);
  }
  
}

void Copypartinfo(Tpartinfo *targetpart,Tpartinfo *sourcepart){
  int alpha;
  targetpart->resinfoptr=sourcepart->resinfoptr;
  for(alpha=0;alpha<4;alpha++) targetpart->p[alpha]=sourcepart->p[alpha];
}

#include "bjorkenboost.cc"
#include "../../random/random.cc"
#include "../../canonical/src/ressubs.cc"
#include "../../staracceptance/staracceptance.cc"
#include "../../na49acceptance/na49acceptance.cc"
#include "balancecalcsubs.cc"

