Tbalance::Tbalance(CParameter *parmset){
  int ibin,alpha;;
  parm=parmset;

  if(parm->staracceptance){
    staracceptance=new Tstaracceptance;
    staracceptance->setup();
  }
  if(parm->na49acceptance){
    na49acceptance=new Tna49acceptance;
  }

  accept=new bool[parm->AMAX];
  for(alpha=0;alpha<5;alpha++){
    Npp[alpha]=new int[parm->nbins];
    Npm[alpha]=new int[parm->nbins];
    Nmm[alpha]=new int[parm->nbins];
    B[alpha]=new double[parm->nbins];
    for(ibin=0;ibin<parm->nbins;ibin++){
      Npp[alpha][ibin]=0;
      Npm[alpha][ibin]=0;
      Nmm[alpha][ibin]=0;
      B[alpha][ibin]=0.0;
    }
  }
  Np=Nm=0;
}

void Tbalance::Calc(){
  int ibin,alpha;
  double x,delx;
  double error,bnorm;
  ofstream resultsfile;
  string filename;
  for(alpha=0;alpha<5;alpha++){
    if(alpha==0){
      filename=parm->resultsdirname+"/balance_qinv.dat\0";
      resultsfile.open(filename.c_str());
    }
    else if(alpha==1)
      resultsfile.open("results/balance_qout.dat");
    else if(alpha==2)
      resultsfile.open("results/balance_qside.dat");
    else if(alpha==3)
      resultsfile.open("results/balance_qlong.dat");
    else if(alpha==4)
      resultsfile.open("results/balance_delY.dat");
    if(alpha<4) delx=parm->delq;
    if(alpha==4) delx=parm->delrap;
    cout << "Np=" << Np << ", Nm=" << Nm << endl;
    resultsfile << "! Np=" << Np << ", Nm=" << Nm << endl;
    cout << "  Q/dY      B        error   |    Npp    Nmm    Npm\n";
    resultsfile << "! Q/dY      B        error   |    Npp    Nmm    Npm\n";
    bnorm=0.0;
    for(ibin=0;ibin<parm->nbins;ibin++){
      B[alpha][ibin]=0.5*double(Npm[alpha][ibin]-2*Npp[alpha][ibin])/double(Np);
      B[alpha][ibin]+=0.5*double(Npm[alpha][ibin]-2*Nmm[alpha][ibin])/double(Nm);
      bnorm+=B[alpha][ibin];
      error=double(Npp[alpha][ibin])/(double(Np)*double(Np))
	+double(Nmm[alpha][ibin])/(double(Nm)*double(Nm))
	+double(Npm[alpha][ibin])*pow(0.5/double(Np)+0.5/double(Nm),2);
      error=sqrt(error);
      x=(0.5+ibin)*delx;
      B[alpha][ibin]=B[alpha][ibin]/delx;
      error=error/delx;
      cout << setw(6) << setprecision(4) << x << " "
	   << setw(10) << setprecision(4) << B[alpha][ibin] << " "
	   << setw(10) << setprecision(4) << error << " | "
	   << setw(6) << Npp[alpha][ibin] << " " << setw(6) << Nmm[alpha][ibin] << " " 
	   << setw(6) << Npm[alpha][ibin] << endl;
      resultsfile << setw(6) << setprecision(4) << x << " "
		  << setw(10) << setprecision(4) << B[alpha][ibin] << " "
		  << setw(10) << setprecision(4) << error << " | "
		  << setw(6) << Npp[alpha][ibin] << " " << setw(6) << Nmm[alpha][ibin] << " " 
		  << setw(6) << Npm[alpha][ibin] << endl;
    }
    cout << "Normalization for B = " << bnorm << endl;
    resultsfile.clear();
    resultsfile.close();
  }
}

void Tbalance::AddParts(int nparts,Tpartinfo **&parts){
  int i,j,ibin,icode,jcode,alpha;
  double qinv,qout,qside,qlong,delY,Ei,Ej,zvertex,x,delx;

  if(parm->staracceptance) zvertex=staracceptance->getzvertex();

  if(parm->acceptall){
    for(i=0;i<nparts;i++){
      icode=parts[i]->resinfoptr->code;
      if(icode==211 || icode==-211){
	accept[i]=1;
	if(icode==211) Np+=1;
	if(icode==-211) Nm+=1;
      }
    }
  }
  else if(parm->staracceptance){
    icode=parts[i]->resinfoptr->code;
    if(icode==211||icode==-211){
      accept[i]=staracceptance->pass(zvertex,parts[i]->p);
      if(accept[i]){
	if(icode==211) Np+=1;
	if(icode==-211) Nm+=1;      
      }
    }
  }
  else if(parm->na49acceptance){
    icode=parts[i]->resinfoptr->code;
    if(icode==211||icode==-211){
      accept[i]=na49acceptance->pass(icode,parts[i]->p);
      if(accept[i]){
	if(icode==211) Np+=1;
	if(icode==-211) Nm+=1;     
      }
    }    
  }
  
  for(i=1;i<nparts;i++){
    icode=parts[i]->resinfoptr->code;
    if(icode==211 || icode==-211){
      if(accept[i]){
	for(j=0;j<i;j++){
	  jcode=parts[j]->resinfoptr->code;
	  if(jcode==211 || jcode==-211){
	    if(accept[j]){
	      Decompose(parts[i]->p,parts[j]->p,
			qinv,qout,qside,qlong,delY);
	      for(alpha=0;alpha<5;alpha++){
		if(alpha==0) x=qinv;
		else if(alpha==1) x=fabs(qout);
		else if(alpha==2) x=fabs(qside);
		else if(alpha==3) x=fabs(qlong);
		else if(alpha==4) x=fabs(delY);
		if(alpha<4) delx=parm->delq;
		if(alpha==4) delx=parm->delrap;
		ibin=int(x/delx);
		if(ibin<parm->nbins){
		  if(icode==211 && jcode==211)
		    Npp[alpha][ibin]+=1;
		  if(icode==-211 && jcode==-211)
		    Nmm[alpha][ibin]+=1;
		  if(icode==211 && jcode==-211)
		    Npm[alpha][ibin]+=1;
		  if(icode==-211 && jcode==211)
		    Npm[alpha][ibin]+=1;
		}
	      }
	    }
	  }
	}
      }
    }
  }
}

// If masses might be different, you should add the 'delmterm' statements
void Tbalance::Decompose(double *pa,double *pb,double &qinv,double &qout,
			 double &qside,double &qlong,double &delY){
  double pt,s,Mlong,roots;
  //double delmterm;
  double ptot[4],q[4],rapa,rapb;
  const int g[4]={1,-1,-1,-1};
  int alpha;
  qinv=0.0;
  s=0.0;
  //delmterm=0.0;
  for(alpha=0;alpha<4;alpha++){
    ptot[alpha]=pa[alpha]+pb[alpha];
    s+=g[alpha]*ptot[alpha]*ptot[alpha];
    //delmterm+=g[alpha]*ptot[alpha]*(pa[alpha]-pb[alpha]);
  }
  //delmterm=delmterm/s;
  for(alpha=0;alpha<4;alpha++){
    q[alpha]=pa[alpha]-pb[alpha];
    //q[alpha]-=ptot[alpha]*delmterm;
    qinv-=g[alpha]*q[alpha]*q[alpha];
  }
  pt=sqrt(ptot[1]*ptot[1]+ptot[2]*ptot[2]);
  Mlong=sqrt(s+pt*pt);
  roots=sqrt(s);

  qside=(ptot[1]*q[2]-ptot[2]*q[1])/pt;
  qlong=(ptot[0]*q[3]-ptot[3]*q[0])/Mlong;
  qout=(roots/Mlong)*(ptot[1]*q[1]+ptot[2]*q[2])/pt;

  qinv=sqrt(qinv);

  rapa=0.5*log((pa[0]+pa[3])/(pa[0]-pa[3]));
  rapb=0.5*log((pb[0]+pb[3])/(pb[0]-pb[3]));
  delY=rapa-rapb;
  if(fabs(qinv*qinv-qside*qside-qout*qout-qlong*qlong)>1.0E-5)
    printf("Failed Test: qinv=%g, q^2=%g\n",
	   qinv,sqrt(qside*qside+qout*qout+qlong*qlong));

}


