// -------------------------------------------------------
// implemented in C++ by Silvio Petriconi
// last modfied: 10/31/2002
// -------------------------------------------------------

#ifndef __CHARGE__H__INCLUDED__
#define __CHARGE__H__INCLUDED__

//----------------------------------------------------------------------
//  This class provides a data type and overloaded operators for 
//  convenient handling of four charges. 

class charge
{
public:
	charge() {};
	charge(int q_is, int q_baryon, int q_strange)
		: qiso(q_is), qb(q_baryon), qs(q_strange) {};

	bool operator == (const charge& Q) const
             { return (Q.qiso==qiso && Q.qb==qb && Q.qs==qs);};
	bool operator < (const charge& Q) const
	     { return (Q.qiso == qiso ? ((Q.qb == qb) ? Q.qs > qs : 
		Q.qb > qb) : Q.qiso > qiso); };

	bool operator > (const charge& Q) const
             { return (!(*this == Q) && !(*this < Q)); };
	bool operator != (const charge& Q) const
	     { return !(*this == Q); };

	void operator += (const charge& Q)
	     { qiso += Q.qiso; qb += Q.qb; qs += Q.qs; };
	void operator *= (int lambda)
	     { qiso *= lambda; qb *= lambda; qs *= lambda; }
	void operator -= (const charge& Q)
	     { qiso -= Q.qiso; qb -= Q.qb; qs -= Q.qs; };

  	int get_qiso() const { return qiso; };
	int get_qb() const { return qb; };
	int get_qs() const { return qs; };
	bool has_negative_charges()
               { return ((qiso < 0) || (qb < 0) || (qs < 0)); };

	int qiso; int qb; int qs; 
};

charge operator -(const charge& a, const charge& b);

charge operator +(const charge& a, const charge& b);


#endif // __CHARGE__H__INCLUDED__
