void resonancedecay(Tpartinfo *&mother,int &nbodies, Tpartinfo **&daughter){
  const double PI=4.0*atan(1.0);
  int ibody,alpha;
  double u[4],mass[4];
  Tresinfo *motherresinfoptr,**daughterresinfoptr;

  double *p[4],kprime[4];
  double e1,e2;
  double q,weight,wmax,sthet,cthet,phi;
  double p3mag,kprimemax,p3max,kprimemax2,kprimemag2,e1prime,e2prime;
  double e12,u12[4];

  daughterresinfoptr=new Tresinfo *[3];
  motherresinfoptr=mother->resinfoptr;

  /* Find the channel and the res info of the daughters */
  //cout << "howdy, try to decay a " << motherresinfoptr->name << endl;
  motherresinfoptr->DecayGetresinfoptr(nbodies,daughterresinfoptr);
  //cout << "nbodies=" << *nbodies << ", Decay products:" << endl;
  //for(ibody=0;ibody<*nbodies;ibody++){
  //cout << "(" << daughterresinfoptr[ibody]->name << ","
  //    << daughterresinfoptr[ibody]->code << ","
  //	    << daughterresinfoptr[ibody]->mass << ")" ;
  //}
  // cout << endl;

  /* Create daughter objects */
  for(ibody=0;ibody<nbodies;ibody++){
    daughter[ibody]->resinfoptr=daughterresinfoptr[ibody];
    mass[ibody+1]=daughterresinfoptr[ibody]->mass;
    for(alpha=0;alpha<4;alpha++) p[ibody+1]=daughter[ibody]->p;
  }
  mass[0]=mother->resinfoptr->mass;

  /* TWO-BODY DECAYS */
  if(nbodies==2){
    q=sqrt(triangle(mass[0],mass[1],mass[2]));
    //cout << "q=" << q << endl;
    cthet=1.0-2.0*ranptr->ran();
    sthet=sqrt(1.0-cthet*cthet);
    phi=2.0*PI*ranptr->ran();
    p[1][3]=q*cthet;
    p[1][1]=q*sthet*cos(phi);
    p[1][2]=q*sthet*sin(phi);
    p[2][3]=-p[1][3];
    p[2][2]=-p[1][2];
    p[2][1]=-p[1][1];
    p[1][0]=sqrt(mass[1]*mass[1]+p[1][1]*p[1][1]
		 +p[1][2]*p[1][2]+p[1][3]*p[1][3]);
    p[2][0]=sqrt(mass[2]*mass[2]+p[2][1]*p[2][1]
		 +p[2][2]*p[2][2]+p[2][3]*p[2][3]);
  }

  /* THREE-BODY DECAYS */
  else if(nbodies==3){
    kprimemax2=triangle(mass[0]-mass[3],mass[1],mass[2]);
    kprimemax=sqrt(kprimemax2);
    p3max=sqrt(triangle(mass[0],mass[1]+mass[2],mass[3]));
    e1=sqrt(pow(mass[1],2)+p3max*p3max);
    e2=sqrt(pow(mass[2],2)+p3max*p3max);
    wmax=p3max*(e1*e2/(mass[1]*mass[2]))*(mass[1]+mass[2])/(e1+e2);
    do{
    TRY_AGAIN:
      do{
	kprime[1]=kprimemax*(2.0*ranptr->ran()-1.0);
	kprime[2]=kprimemax*(2.0*ranptr->ran()-1.0);
	kprime[3]=kprimemax*(2.0*ranptr->ran()-1.0);
	kprimemag2=kprime[1]*kprime[1]+
	  kprime[2]*kprime[2]+kprime[3]*kprime[3];
      } while(kprimemag2>kprimemax2);
      e1prime=sqrt(kprimemag2+mass[1]*mass[1]);
      e2prime=sqrt(kprimemag2+mass[2]*mass[2]);
      if(e1prime+e2prime+mass[3]>mass[0]) goto TRY_AGAIN;
      p3mag=sqrt(triangle(mass[0],e1prime+e2prime,mass[3]));
      cthet=1.0-2.0*ranptr->ran();
      sthet=sqrt(1.0-cthet*cthet);
      phi=2.0*PI*ranptr->ran();
      p[3][3]=p3mag*cthet;
      p[3][1]=p3mag*sthet*cos(phi);
      p[3][2]=p3mag*sthet*sin(phi);
      p[3][0]=sqrt(p3mag*p3mag+mass[3]*mass[3]);
      e12=sqrt(pow(e1prime+e2prime,2)+p3mag*p3mag);
      for(alpha=1;alpha<4;alpha++) u12[alpha]=-p[3][alpha]/(e1prime+e2prime);
      u12[0]=sqrt(1.0+u12[1]*u12[1]+u12[2]*u12[2]+u12[3]*u12[3]);
      kprime[0]=e1prime;
      lorentz(u12,kprime,p[1]);
      kprime[0]=e2prime;
      for(alpha=1;alpha<=3;alpha++) kprime[alpha]=-kprime[alpha];
      lorentz(u12,kprime,p[2]);
      weight=p3mag*(p[1][0]*p[2][0]/(e1prime*e2prime))
	*((e1prime+e2prime)/(p[1][0]+p[2][0]));
    } while(ranptr->ran()<weight/wmax);
  }
  
  /* Boost the new particles */
  for(alpha=0;alpha<4;alpha++)
    u[alpha]=mother->p[alpha]/mother->resinfoptr->mass;
  for(ibody=1;ibody<=nbodies;ibody++){
    lorentz(u,p[ibody],daughter[ibody-1]->p);
  }

  delete [] daughterresinfoptr;

  //cout << "finishing decay\n";
}



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

void lorentz(double *u,double *p1,double *p1prime){
  static double n[4]={1.0,0.0,0.0,0.0};
  static double g[4]={1.0,-1.0,-1.0,-1.0};
  int mu;
  double udotn=0.0,p1dotn=0.0,p1dotu=0.0;
  for(mu=0;mu<4;mu++){
    p1dotn=p1dotn+p1[mu]*n[mu]*g[mu];
    p1dotu=p1dotu+p1[mu]*u[mu]*g[mu];
    udotn=udotn+u[mu]*n[mu]*g[mu];
  }
  for(mu=0;mu<4;mu++){
    p1prime[mu]=-((p1dotu+p1dotn)/(1.0+udotn))*(n[mu]+u[mu])
      +2.0*p1dotn*u[mu]+p1[mu];
  }
}

double triangle(double m0,double m1,double m2){
  double answer,m0sq,m1sq,m2sq;
#ifdef EXTRA_TESTS
  if(m0<m1+m2) {
    printf("Disaster with triangle\n");
    exit(1);
  }
#endif
  m0sq=m0*m0;m1sq=m1*m1;m2sq=m2*m2;
  answer=m0sq*m0sq+m1sq*m1sq+m2sq*m2sq;
  answer=answer-2.0*(m0sq*m1sq+m0sq*m2sq+m1sq*m2sq);
  answer=answer/(4.0*m0sq);
  return answer;
}

void WriteParts(ofstream &outfile,int &nparts,Tpartinfo **&newparts){
  int ipart;
  outfile << nparts << endl;
  for(ipart=0;ipart<nparts;ipart++){
    outfile << newparts[ipart]->resinfoptr->code << " "
	    << newparts[ipart]->p[1] << " "
	    << newparts[ipart]->p[2] << " "
	    << newparts[ipart]->p[3] << endl;
  }
}

void ReadAndDecay(ifstream &inputfile,int AMAX,
		  int &nparts,Tpartinfo **&newparts,Treslist &reslist){
  Tpartinfo **mother;
  Tpartinfo **daughter;
  int nread,iread,nmothers;
  double ptest[4]={0.0};
  int NPARTMAX,mothercode,nbodies,ibody,idecay,ipart,alpha;
  
  mother=new Tpartinfo *[10];
  daughter=new Tpartinfo *[10];
  for(ipart=0;ipart<10;ipart++){
    mother[ipart]=new Tpartinfo();
    daughter[ipart]=new Tpartinfo();
  }

  inputfile >> nread;
  nparts=0;
  for(iread=0;iread<nread;iread++){
    inputfile >> mothercode >> mother[0]->p[1] >> mother[0]->p[2] 
	      >> mother[0]->p[3];
    reslist.Getresinfoptr(mothercode,mother[0]->resinfoptr);
    mother[0]->p[0]=pow(mother[0]->resinfoptr->mass,2);
    for(alpha=1;alpha<4;alpha++) mother[0]->p[0]+=pow(mother[0]->p[alpha],2);
    mother[0]->p[0]=sqrt(mother[0]->p[0]);
    nbodies=1;
    if(mother[0]->resinfoptr->decay==0){
      Copypartinfo(newparts[nparts],mother[0]);
      nparts+=1;
      if(nparts>AMAX){
	cout << "Inside ReadAndDecay, nparts too big, increase AMAX!\n";
	cout << "nparts=" << nparts << endl;
	exit(1);
      }
    }
    else{
      nmothers=1;
      while(nmothers>0){
	nmothers-=1;
	resonancedecay(mother[nmothers],nbodies,daughter);
	for(ibody=0;ibody<nbodies;ibody++){
	  if(daughter[ibody]->resinfoptr->decay==0){
	    Copypartinfo(newparts[nparts],daughter[ibody]);
	    nparts+=1;
	    if(nparts>AMAX){
	      cout << "Inside ReadAndDecay, nparts too big, increase AMAX!\n";
	      cout << "nparts=" << nparts << endl;
	      exit(1);
	    }
	  }
	  else{
	    Copypartinfo(mother[nmothers],daughter[ibody]);
	    nmothers+=1;
	  }
	}
      }
    
    }
  }

  for(ipart=0;ipart<10;ipart++){
    delete mother[ipart];
    delete daughter[ipart];
  }
  delete [] mother;
  delete [] daughter;

}

void Copypartinfo(Tpartinfo *targetpart,Tpartinfo *sourcepart){
  int alpha;
  //cout << "copying " << sourcepart->resinfoptr->name << endl;
  targetpart->resinfoptr=sourcepart->resinfoptr;
  for(alpha=0;alpha<4;alpha++) targetpart->p[alpha]=sourcepart->p[alpha];
}

