//===========================================================
//  PARTICLEZOO implementation

int particle_zoo::getMaxA() const
{
   int A = 0;
   const_iterator i;
   for (i = begin(); i != end(); i++)
     A = ((A > (*i).its_A) ? A : (*i).its_A);
   return A;
}

void particle_zoo::read_from_file(string filename, int skiplines)
{
  ifstream infile(filename.c_str());
  if (!infile.good()) throw XFileNotFound();
  char buffer[0xffff];
  while (skiplines > 0)
    {
      infile.getline(buffer, sizeof(buffer)); skiplines--;
    } 

  int ID, QE, QB, QS, BStable; double QSpin, DWidth, Mass;
  string Name;
  particle P;

  while (!infile.eof())
    {
      infile >> ID; infile >> Mass; infile >> QE; infile >> QB; 
      infile >> QS; 
      infile >> QSpin; 
      cout << "Mass=" << Mass << endl;
      if (infile.eof()) break;  // added because last line is not recognized
                                // otherwise... (iostream bug?)
      infile >> BStable; 
      infile >> DWidth; 
      infile >> Name;
      infile.getline(buffer, sizeof(buffer));

      int Isospin2 = 2*QE - QS - QB; 
      P.its_ID = ID;
      P.its_Mass = Mass;
      P.its_width=DWidth;
      P.its_A = (BStable == 1 ? 2 : 1);
      P.its_Name = Name;
      P.its_charge = charge(Isospin2, QB, QS);
      this->push_back(P);
      //#ifdef _DEBUGMSG_
      cout << " added " << Name << " isospin " << (double)Isospin2/2 << " A=" << P.its_A << endl;
      //#endif
    }
  update_partitions(); 
}

void particle_zoo::update_partitions()
{
    particle_zoo::iterator i;
    for (i = begin(); i != end(); i++)
    {
       i->its_W = getomega(i->its_Mass, i->its_width, i->its_ID, 
			   env.its_temperature, env.its_volume);
 #ifdef _DEBUGMSG_
	cout << "particle " << i->its_Name << " partition function: " << i->its_W << endl;
  #endif
    }
}

void particle_zoo::set_env_conditions(const environment& env_cond)
{
   env = env_cond;
   update_partitions();
}

const environment& particle_zoo::get_env_conditions() const
{
   return env;
}

