// program to take hijing, bjorken, or random events and filter them
// through STAR's vertex pseudorapidity survival distributions - fast eta sim.
// Marguerite Belt Tonjes, belt@mac.com, 01/21/02
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>

/* Prototypes */
float etabin[10][100];
float etahist[10][100];
float normetahist[10][100];
float ntot;
long IDUM;
double ran2(void);
void define_etahist();
int vertcut (float eta, int vbin, float rancomp);
// eta boundaries
const double etaabsmax=1.3;
const double etaabsmin=-1.3;
int keeppid[10000],pass;
float keepp[10000],keeppt[10000],keepeta[10000],keepdedx[10000];

// main program
int main()
{
  using namespace std;
  FILE *fp;
  FILE *fh;
  FILE *fpo;
  FILE *fho;
  int passed,myevt,myrun,ngoodtrack,myntrack,evt,ntrack,nfiles;
  int pid,ie,maxev,cbin,vbin,itrk,counter,ifile,ibin,ik,i;
  int inev[96],pause;
  float myvert,dedx,p,pt,eta,alletamax;
  double rancomp,rancomp2;
  char filename[100],*inrun[96],hfile[80],ofilename[100],ohfile[100];
  char *outrun[96],*outdir[3],afile[80];
  double vproblow[10],vprobhigh[10],vmiddle[10];
  string ptype[10];
  ptype[0]="0";
  ptype[1]="1";
  ptype[2]="2";
  ptype[3]="3";
  ptype[4]="4";
  ptype[5]="5";
  ptype[6]="6";
  ptype[7]="7";
  ptype[8]="8";
  ptype[9]="9";
  // set some parameters
  // these are based on a fit to 14,579 130 GeV AuAu STAR minbias events sorted by vertex
  // bin, with bins from -75 to +75. 
  vproblow[0]=0.00;     vprobhigh[0]=0.081848;
  vproblow[1]=0.081848; vprobhigh[1]=0.178738;
  vproblow[2]=0.178738; vprobhigh[2]=0.274408;
  vproblow[3]=0.274408; vprobhigh[3]=0.378478;
  vproblow[4]=0.378478; vprobhigh[4]=0.485598;
  vproblow[5]=0.485598; vprobhigh[5]=0.596718;
  vproblow[6]=0.596718; vprobhigh[6]=0.705598;
  vproblow[7]=0.705598; vprobhigh[7]=0.809268;
  vproblow[8]=0.809268; vprobhigh[8]=0.906023;
  vproblow[9]=0.906023; vprobhigh[9]=1.00;
  vmiddle[0]= -62.5;
  vmiddle[1]= -47.5;
  vmiddle[2]= -32.5;
  vmiddle[3]= -17.5;
  vmiddle[4]= -2.5;
  vmiddle[5]= 12.5;
  vmiddle[6]= 27.5;
  vmiddle[7]= 42.5;
  vmiddle[8]= 57.5;
  vmiddle[9]= 72.5;
  //
  alletamax=1.3;
  define_etahist();
  outdir[0]="/bjorken/";
  inrun[0]="/bjorken/bjk_t9n50i225f120";
  inev[0]=100000;
  outrun[0]="bjkt9n50i2235f120";
  strcpy(filename,inrun[ifile]);
  strcat(filename,".bin");
  fp=fopen(filename,"rb");
  strcpy(hfile,inrun[ifile]);
  strcat(hfile,"_header.bin");
  fh=fopen(hfile,"rb");
  if(fp==NULL){
    cout<<filename<<" cannot be found, error"<<endl;
    exit(1);
  }
  if(fh==NULL){
    cout<<hfile<<" cannot be found, error"<<endl;
    exit(1);
  }
  strcpy(ofilename,outdir[ifile]);
  strcat(ofilename,outrun[ifile]);
  strcat(ofilename,"ft.bin");
  fpo=fopen(ofilename,"wb");
  strcpy(ohfile,outdir[ifile]);
  strcat(ohfile,outrun[ifile]);
  strcat(ohfile,"ft_header.bin");
  fho=fopen(ohfile,"wb");
  if(fpo==NULL){
    cout<<ofilename<<" cannot be found, error"<<endl;
    exit(1);
  }
  if(fho==NULL){
    cout<<ohfile<<" cannot be found, error"<<endl;
    exit(1);
  } 
  maxev=inev[ifile];
  //* end setting up input files
  //* begin reading files and filling particle arrays
  // begin event loop
  counter=0;
  for(ie=0;ie<maxev;ie++){
    fread(&passed,sizeof(int),1,fh);
    fread(&myevt,sizeof(int),1,fh);
    fread(&myrun,sizeof(int),1,fh);
    fread(&ngoodtrack,sizeof(int),1,fh);
    fread(&myntrack,sizeof(int),1,fh);
    fread(&myvert,sizeof(float),1,fh);
    fread(&evt,sizeof(int),1,fp);
    fread(&ntrack,sizeof(int),1,fp);
    // zero index of positive and negative particles for this event
    // also zero "keep" index for use in denominator
    counter++;
    if(counter==1000){
      cout<<"Reading event "<<ie+1<<" file: "<<ifile<<endl;
      counter=0;
    }
    // random call for vertex bin (based on actual event distribution)
    rancomp=0.;
    rancomp=ran2();
    for(ibin=0;ibin<10;ibin++){
      if(rancomp>=vproblow[ibin] && rancomp<vprobhigh[ibin]){
	vbin=ibin;
	myvert=vmiddle[ibin];
      }
    }
    // will assume all these are central (can do non central)
    cbin=0;
    ik=0;
    // begin track reading loop
    for(itrk=0;itrk<ntrack;itrk++){
      fread(&pid,sizeof(int),1,fp);
      fread(&dedx,sizeof(float),1,fp);
      fread(&p,sizeof(float),1,fp);
      fread(&pt,sizeof(float),1,fp);
      fread(&eta,sizeof(float),1,fp);
      rancomp=0.;
      pass=0;
      if(eta<=1.3 && eta>=-1.3){
	// if particle survives, write it to matrix, will write to file afterwards
	// common filter for all events
	if(pt>0.1 && pt<=2.){ 
	  // PID cuts filter, if outside of momentum range, becomes an "11", unknown 
	  if(pid==2 && p>0.7){pid=11;}
	  else if(pid==3 && p>0.8){pid=11;}
	  else if(pid==4 && p>1.0){pid=11;}
	  if(pid==-2 && p>0.7){pid=-11;}
	  else if(pid==-3 && p>0.8){pid=-11;}
	  else if(pid==-4 && p>1.0){pid=-11;}
	  rancomp=ran2();
	  pass=vertcut(eta,vbin,rancomp);
	} else { pass=0;}
	// end checking |eta|<1.3
      }else { pass=0;}
      // keep particles that pass and event information to write to file correctly.
      if(pass==1){
	rancomp2=ran2();
	if(rancomp2<=1.00){
	  keeppid[ik]=pid;
	  keepp[ik]=p;
	  keeppt[ik]=pt;
	  keepeta[ik]=eta;
	  keepdedx[ik]=dedx;
	  ik=ik+1;
	}
      }
      // end reading tracks
    }
    myntrack=ik;
    // write event info to file. Be sure to have changed myntrack, myvert, and ntrack
    fwrite(&passed,sizeof(int),1,fho);
    fwrite(&myevt,sizeof(int),1,fho);
    fwrite(&myrun,sizeof(int),1,fho);
    fwrite(&ngoodtrack,sizeof(int),1,fho);
    fwrite(&myntrack,sizeof(int),1,fho);
    fwrite(&myvert,sizeof(float),1,fho);
    fwrite(&evt,sizeof(int),1,fpo);
    fwrite(&myntrack,sizeof(int),1,fpo);
    for(itrk=0;itrk<myntrack;itrk++){
      fwrite(&keeppid[itrk],sizeof(int),1,fpo);
      fwrite(&keepdedx[itrk],sizeof(float),1,fpo);
      fwrite(&keepp[itrk],sizeof(float),1,fpo);
      fwrite(&keeppt[itrk],sizeof(float),1,fpo);
      fwrite(&keepeta[itrk],sizeof(float),1,fpo);
    }
    // end reading events
  }
  // close the open files
  fclose(fp);
  fclose(fh);
  fclose(fpo);
  fclose(fho); 
}
void define_etahist(){
  using namespace std;
  int i,cbin,inbin,vbin;
  float binsize;
  float eta,dndeta,dndetanorm;
  char someline[80],aline[80],filename[80];
  // 4 centrality bins
  // 10 vertex bins
  char *cent[4];
  cent[0]="0";
  cent[1]="1";
  cent[2]="2";
  cent[3]="3";
  char *ind[10];
  ind[0]="0";
  ind[1]="1";
  ind[2]="2";
  ind[3]="3";
  ind[4]="4";
  ind[5]="5";
  ind[6]="6";
  ind[7]="7";
  ind[8]="8";
  ind[9]="9";
  // currently setup to do only 1 centrality bin cbin=0;
  cbin=0;
  for(vbin=0;vbin<10;vbin++){
    strcpy(filename,"/hist/etahist_c");
    strcat(filename,cent[cbin]);
    strcat(filename,"_v");
    strcat(filename,ind[vbin]);
    strcat(filename,".txt");
    ifstream infile(filename,ios::in);
    if(!infile){
      cerr<<"Input File/location: "<<filename<<" cannot be found"<<endl;
      exit(1);
    }
    binsize=3./100.;
    inbin=0;
    ntot=0.;
    for(i=0;i<100;i++){
      infile>>eta>>dndeta>>dndetanorm;
      etabin[vbin][i]=eta;
      normetahist[vbin][i]=dndetanorm;
    }
    infile.close();
    // normetahist is the probability of a particle surviving in a given dN/deta
  }
  cout<<"defined etahist"<<endl;
  return;
}
int vertcut(float eta, int vbin, float rancomp){
  // compare input eta to the edges of the bins.
  using namespace std;
  float etalow,etahigh;
  int looping,i,gotbin;
  int pause;
  etalow=-1.5;
  looping=0;
  i=0;
  while(looping!=1){
    etahigh=etalow+0.03;
    if(eta>=etalow && eta<etahigh){
      if(rancomp<=normetahist[vbin][i]){
	pass=1;
	gotbin=i;
      } else if(rancomp>normetahist[vbin][i]){
	pass=0;
      }
      looping=1;
    } else {
      etalow=etahigh;
    }
    i++;
  }
  return pass;
}
/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */

/* long IDUM=-123 */
/* double ran2(void); */
double gauss(void);
/* To use this routine, call the ran2 with the form */
/*x=ran2();*/
/* and x will be a random number between 0 and 1, which won't repeat */
/* before every 10^18 numbers */
/* To initialize, call with IDUM being a NEGATIVE number */
/* This came out of the ran2 function from numerical recipes */

#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

double ran2() {
  using namespace std;
  long J;
  long K;
  static long IDUM2=123456789;
  static long IY=0;
  static long IV[NTAB];
  double tempguy;
  if(IDUM <=0){
    if(-(IDUM)<1) IDUM=1;
    else IDUM = -(IDUM);
    IDUM2=(IDUM);
    for (J=NTAB+7;J>=0;J--) {
      K=(IDUM)/IQ1;
      IDUM=IA1*(IDUM-K*IQ1)-K*IR1;
      if(IDUM<0) IDUM +=IM1;
      if(J<NTAB) IV[J]=IDUM;
    }
    IY=IV[0];
    //printf("Initialization Completed %d\n",IDUM);
  }
  K=(IDUM)/IQ1;
  IDUM=IA1*(IDUM-K*IQ1)-K*IR1;
  if(IDUM<0) IDUM+=IM1;
  K=IDUM2/IQ2;
  IDUM2=IA2*(IDUM2-K*IQ2)-K*IR2;
  if(IDUM2<0) IDUM2+=IM2;
  J=IY/NDIV;
  IY=IV[J]-IDUM2;
  IV[J]=IDUM;
  if(IY<1) IY+=IMM1;
  tempguy=AM*IY;
  if(tempguy>RNMX) return RNMX;
  else return tempguy;
}
#undef IM1
#undef IM2
#undef AM
#undef IMM1
#undef IA1
#undef IA2
#undef IQ1
#undef IQ2
#undef IR1
#undef IR2
#undef NTAB
#undef NDIV
#undef EPS
#undef RNMX

double gauss(){
  using namespace std;
  double x,y,r2,r,c,randy;
 TRY_AGAIN:
  x=1.0-2.0*ran2();
  y=1.0-2.0*ran2();
  r2=x*x+y*y;
  if(r2>1.0) goto TRY_AGAIN;
  r=sqrt(r2);
  c=x/r;
  randy=c*sqrt(-2.0*log(r2));
  return randy;
}
